/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * NotAllAtomsOfMoleculeAction.cpp
 *
 *  Created on: May 12, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "Descriptors/AtomSelectionDescriptor.hpp"
#include "Descriptors/AtomOfMoleculeSelectionDescriptor.hpp"
#include "Atom/atom.hpp"
#include "molecule.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "World.hpp"

#include <boost/foreach.hpp>
#include <iostream>
#include <string>

#include "NotAllAtomsOfMoleculeAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "NotAllAtomsOfMoleculeAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
ActionState::ptr SelectionNotAllAtomsOfMoleculeAction::performCall() {
  LOG(1, "Unselecting all atoms of currently selected molecules.");
  std::vector<const atom *> unselectedAtoms = const_cast<const World &>(World::getInstance()).
      getAllAtoms(!AtomsBySelection());
  World::getInstance().unselectAllAtoms(AtomsByMoleculeSelection());
  LOG(0, World::getInstance().countSelectedAtoms() << " atoms remain selected.");
  return ActionState::ptr(new SelectionNotAllAtomsOfMoleculeState(unselectedAtoms, params));
}

ActionState::ptr SelectionNotAllAtomsOfMoleculeAction::performUndo(ActionState::ptr _state) {
  SelectionNotAllAtomsOfMoleculeState *state = assert_cast<SelectionNotAllAtomsOfMoleculeState*>(_state.get());

  World::getInstance().selectAllAtoms(AllAtoms());
  BOOST_FOREACH(const atom *_atom, state->unselectedAtoms)
    World::getInstance().unselectAtom(_atom);

  return ActionState::ptr(_state);
}

ActionState::ptr SelectionNotAllAtomsOfMoleculeAction::performRedo(ActionState::ptr _state){
  //SelectionNotAllAtomsOfMoleculeState *state = assert_cast<SelectionNotAllAtomsOfMoleculeState*>(_state.get());

  World::getInstance().unselectAllAtoms(AtomsByMoleculeSelection());

  return ActionState::ptr(_state);
}

bool SelectionNotAllAtomsOfMoleculeAction::canUndo() {
  return true;
}

bool SelectionNotAllAtomsOfMoleculeAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
